﻿# 変数設定
$hostname = 'tr-e-it-api.obpm-neo-demo.com' # API の URL ホスト名
$tenant_id = 'd6abb579-fba2-4700-9ab7-f89f5dd996e4' # アプリのテナント ID
$client_id = '4a573796-87d9-4014-ac36-f90023215a22' # アプリのクライアント ID
$client_secret = 'FNq8Q~GGP06f74sLo5yReGM6XN3gvCgXksemOcq9' # クライアントシークレット

$pjPatternName = '管理対象プロジェクト（部門）' # 「一覧：プロジェクト一覧」で使用する条件パターン名（※PJコード枝番を表示させること）
$problemPatternName = '課題データ取込対応パターン' # 「一覧：課題一覧」で使用する条件パターン名

# 処理開始
$keyword = Read-Host "件名の検索キーワードを入力できます（省略可）" # 障害検索キーワード（対象：件名）

# カレントディレクトリをスクリプトのディレクトリに変更する 
$scriptPath = $MyInvocation.MyCommand.Path
$scriptDirectory = Split-Path $scriptPath
Set-Location -Path $scriptDirectory

#region トークン取得

$url = 'https://login.microsoftonline.com/' + $tenant_id + '/oauth2/token' # トークン取得のためにアクセスする AAD 認証の URL
$body = @{
    grant_type = 'client_credentials'
    resource = $client_id
    client_id = $client_id
    client_secret = $client_secret
}

$result = $null
$result = Invoke-RestMethod -Method Post -Body $body $url # $result.access_token にアクセストークンが格納されている

# Write-Output($result); # デバッグ用: レスポンスの中身を確認する
# Set-Content -Path token.txt $result.access_token # デバッグ用: アクセストークンをファイルに出力する

#endregion

$header = @{
    'Cache-Control' = 'no-cache'
    'Authorization' = 'Bearer ' + $result.access_token
}
$apiHost = 'https://' + $hostname # API の URL

# 出力する配列のキーを指定
$key = "data"

#region API 実行 その１（データ種類取得）

$apiUrl1 = $apiHost + '/api/v1/export/kinds' # データ種類を全件取得します

# API実行
$apiResult1 = $null
$apiResult1 = Invoke-RestMethod -Method Get -Headers $header $apiUrl1

# Write-Output(ConvertTo-JSON $apiResult1) # デバッグ用：レスポンスの中身を確認する

# 指定されたキーの配列データを取得
$kinds = $apiResult1.$key

# 今回使用するデータ種類の抽出
$kind1 = $kinds | Where-Object {$_.kindName -eq "一覧：プロジェクト一覧"} | Select-Object -First 1
$kind2 = $kinds | Where-Object {$_.kindName -eq "一覧：課題一覧"} | Select-Object -First 1

#endregion

#region API 実行 その２（条件パターン取得）

$apiUrl2 = $apiHost + '/api/v1/export/patterns?kindId=' + $kind1.kindId  # 条件パターンを取得します

# API実行
$apiResult2 = $null
$apiResult2 = Invoke-RestMethod -Method Get -Headers $header $apiUrl2

# 指定されたキーの配列データを取得
$pjKindPatterns = $apiResult2.$key

$apiUrl3 = $apiHost + '/api/v1/export/patterns?kindId=' + $kind2.kindId  # 条件パターンを取得します

# API実行
$apiResult3 = $null
$apiResult3 = Invoke-RestMethod -Method Get -Headers $header $apiUrl3

# 指定されたキーの配列データを取得
$questionsKindPatterns = $apiResult3.$key

# 今回使用する条件パターンの抽出
$pattern1 = $pjKindPatterns | Where-Object {$_.patternName -eq $pjPatternName} | Select-Object -First 1
$pattern2 = $questionsKindPatterns | Where-Object {$_.patternName -eq $problemPatternName} | Select-Object -First 1

#endregion

#region API 実行 その３（プロジェクト一覧取得）
$apiUrl4 = $apiHost + $pattern1.request # 最初に設定した「一覧：プロジェクト一覧」の条件パターンを使用します

# API実行
$apiResult4 = $null
$apiResult4 = Invoke-RestMethod -Method Get -Headers $header $apiUrl4

# Write-Output(ConvertTo-JSON $apiResult1) # デバッグ用：レスポンスの中身を確認する

# 出力する配列のキーを指定
$key = "data"

# 指定されたキーの配列データを取得
$projects = $apiResult4.$key

#endregion

#region API 実行 その４（プロジェクトごとに課題一覧取得）

#配列初期化
$outputDataItems = @()

foreach ($project in $projects) {
    $pjCodeNo = $project.ＰＪコード枝番
    $apiUrl5 = $apiHost + $pattern2.request + '/' + $pjCodeNo # 最初に設定した「一覧：課題一覧」の条件パターンを使用します

    $apiResult5 = $null
    $apiResult5 = Invoke-RestMethod -Method Get -Headers $header $apiUrl5

    $problems = $apiResult5.$key

    # 件名でキーワード検索をして絞り込みます
    $keywordMatchedProblems = $problems | Where-Object {$_.件名 -like '*' + $keyword + '*' -or $keyword -eq ''}

    foreach ($problem in $keywordMatchedProblems ) {
        $problem | Add-Member -NotePropertyName "PJコード枝番" -NotePropertyValue $project.ＰＪコード枝番
        $problem | Add-Member -NotePropertyName "PJ名称" -NotePropertyValue $project.ＰＪ名称
        $problem | Add-Member -NotePropertyName "PJ状況" -NotePropertyValue $project.ＰＪ状況
        $problem | Add-Member -NotePropertyName "PM担当者" -NotePropertyValue $project.PM担当者
        $problem | Add-Member -NotePropertyName "PL担当者" -NotePropertyValue $project.PL担当者
        $outputDataItems += @($problem)
    }
}

# Write-Output(ConvertTo-JSON $outputDataItems) # デバッグ用：CSVの元となるデータ配列を確認する

#endregion

#region CSV出力

$csvFilePath = "管理PJの課題一覧.csv"
$outputDataItems | Export-Csv -Path $csvFilePath -NoTypeInformation -Encoding utf8

#endregion