﻿# 変数設定
$hostname = 'tr-e-it-api.obpm-neo-demo.com' # API の URL ホスト名
$tenant_id = 'd6abb579-fba2-4700-9ab7-f89f5dd996e4' # アプリのテナント ID
$client_id = '4a573796-87d9-4014-ac36-f90023215a22' # アプリのクライアント ID
$client_secret = 'FNq8Q~GGP06f74sLo5yReGM6XN3gvCgXksemOcq9' # クライアントシークレット

$pattanName1 = '【CSウェビナーデモ用】予定原価' # 条件パターン名
$pattanName2 = '【CSウェビナーデモ用】実績工数' # 条件パターン名
$pattanName3 = '【CSウェビナーデモ用】アカウント情報' # 条件パターン名
$pattanName4 = '【CSウェビナーデモ用】工数委託者単価' # 条件パターン名

$monthDays = 20 # 月の営業日数
$currentDays = 17 # 現時点での稼働日数
$monthHours = 160 # 1人月あたりの時間

# 社員ランクごとのコスト
$rankCost = @{
    役員 = 1412000
    管理職ＰＭ = 1270000
    管理職ＰＬ = 1060000
    総合職Ａ = 985000
    総合職Ｂ = 868000
    総合職Ｃ = 553000
}

# 処理開始

# カレントディレクトリをスクリプトのディレクトリに変更する 
$scriptPath = $MyInvocation.MyCommand.Path
$scriptDirectory = Split-Path $scriptPath
Set-Location -Path $scriptDirectory

#region トークン取得

$url = 'https://login.microsoftonline.com/' + $tenant_id + '/oauth2/token' # トークン取得のためにアクセスする AAD 認証の URL
$body = @{
    grant_type = 'client_credentials'
    resource = $client_id
    client_id = $client_id
    client_secret = $client_secret
}

$result = $null
$result = Invoke-RestMethod -Method Post -Body $body $url # $result.access_token にアクセストークンが格納されている

# Write-Output($result); # デバッグ用: レスポンスの中身を確認する
# Set-Content -Path token.txt $result.access_token # デバッグ用: アクセストークンをファイルに出力する

#endregion

#region API 実行 その１（使用する条件パターン選択）

$header = @{
    'Cache-Control' = 'no-cache'
    'Authorization' = 'Bearer ' + $result.access_token
}

$apiHost = 'https://' + $hostname # API Management の URL
$apiUrl = $apiHost + '/api/v1/export/patterns' # 「一覧：プロジェクト一覧」の「管理対象プロジェクト（部門）」を使用します

# API実行
$apiResult = $null
$apiResult = Invoke-RestMethod -Method Get -Headers $header $apiUrl

# Write-Output(ConvertTo-JSON $apiResult) # デバッグ用：レスポンスの中身を確認する

#endregion

#region 各種条件パターン抽出

# 出力する配列のキーを指定
$key = "data"

# 指定されたキーの配列データを取得
$dataArray = $apiResult.$key

# 予定原価取得 の条件パターン
$pattern1 = $dataArray | Where-Object {$_.patternName -eq $pattanName1} | Select-Object -First 1
# 実績工数取得 の条件パターン
$pattern2 = $dataArray | Where-Object {$_.patternName -eq $pattanName2} | Select-Object -First 1
# アカウント情報取得 の条件パターン
$pattern3 = $dataArray | Where-Object {$_.patternName -eq $pattanName3} | Select-Object -First 1
# 工数委託者の単価取得 の条件パターン
$pattern4 = $dataArray | Where-Object {$_.patternName -eq $pattanName4} | Select-Object -First 1

#endregion

#region API 実行 その２（抽出した条件パターンで出力）

# 予定原価取得
$apiUrl = $apiHost + $pattern1.request

$apiResult = $null
$apiResult = Invoke-RestMethod -Method Get -Headers $header $apiUrl

$planCosts = $apiResult.$key

# 実績工数取得
$apiUrl = $apiHost + $pattern2.request

$apiResult = $null
$apiResult = Invoke-RestMethod -Method Get -Headers $header $apiUrl

$actualManHours = $apiResult.$key

# アカウント情報取得
$apiUrl = $apiHost + $pattern3.request

$apiResult = $null
$apiResult = Invoke-RestMethod -Method Get -Headers $header $apiUrl

$accounts = $apiResult.$key

# 工数委託者の単価取得
$apiUrl = $apiHost + $pattern4.request

$apiResult = $null
$apiResult = Invoke-RestMethod -Method Get -Headers $header $apiUrl

$contCosts = $apiResult.$key

#endregion

#region データ加工

# 人ごと、プロジェクトごとの実績工数から金額を算出する
foreach($actualManHour in $actualManHours)
{
    # 工数のアカウントコードから自社 & 委託契約の情報を取得
    $account = $accounts | Where-Object {$_.アカウントコード -eq $actualManHour.アカウント} | Select-Object -First 1
    $contCost = $contCosts | Where-Object {$_.アカウントコード -eq $actualManHour.アカウント} | Select-Object -First 1

    # 委託契約情報があれば、契約金額を単価とする
    If($null -ne $contCost)
    {
        $actualManHour | Add-Member -NotePropertyName "基準単価" -NotePropertyValue $contCost."当月契約＿契約金額"
    }
    # 自社アカウントであれば、社員ランク名と対応する単価を設定する
    ElseIf($null -ne $account)
    {
        $actualManHour | Add-Member -NotePropertyName "基準単価" -NotePropertyValue $rankCost.($account.社員ランク名)
    }
    # 設定できない場合、0とする
    Else
    {
        $actualManHour | Add-Member -NotePropertyName "基準単価" -NotePropertyValue 0
    }

    # 基準単価をベースに、原価金額を工数から計算する
    $actualManHour | Add-Member -NotePropertyName "現時点の参考原価金額" -NotePropertyValue ([Math]::Round($actualManHour."工数（時間）" / $monthHours * $actualManHour.基準単価))
}

# 人ごと、プロジェクトごとの金額をプロジェクト単位で集計する
foreach($planCost in $planCosts)
{
    $planCost | Add-Member -NotePropertyName "予定労務費と工数委託費の合計" -NotePropertyValue ($planCost.予定原価_労務費 + $planCost.予定原価_工数委託費)

    # アカウント、プロジェクトごとに算出した参考原価金額を合計する
    $planCost | Add-Member -NotePropertyName "参考原価金額" -NotePropertyValue 0
    $filteredactualManHours = $actualManHours | Where-Object {$_.ＰＪコード枝番 -eq $planCost.ＰＪコード枝番}
    foreach($actualManHour in $filteredactualManHours) {
        $planCost.参考原価金額 += $actualManHour.現時点の参考原価金額
    }

    # 現時点で使われる想定の原価と実際の原価の差異を計算する
    $planCost | Add-Member -NotePropertyName "当日時点の差異" -NotePropertyValue ([Math]::Round($planCost.参考原価金額 - $planCost.予定労務費と工数委託費の合計 * $currentDays / $monthDays))
    # 以降は計画どおりに進むとして、当月の原価がどうなるかを計算する
    $planCost | Add-Member -NotePropertyName "当月予測" -NotePropertyValue ($planCost.予定労務費と工数委託費の合計 + $planCost.当日時点の差異)
    # 現時点までの実績をベースに原価計算すると、当月の原価がどうなるのか計算する
    $planCost | Add-Member -NotePropertyName "当日時点の日割原価のまま月末までいった場合" -NotePropertyValue ([Math]::Round($planCost.参考原価金額 * $monthDays / $currentDays))
    # 実績ベースで進んだ場合の原価差異を計算する
    $planCost | Add-Member -NotePropertyName "当月予測の差異" -NotePropertyValue ($planCost.予定労務費と工数委託費の合計 - $planCost.当日時点の日割原価のまま月末までいった場合)
}

#endregion

#region CSV出力

$csvFilePath1 = "プロジェクト別月中超過原価確認.csv"
$planCosts | Export-Csv -Path $csvFilePath1 -NoTypeInformation -Encoding utf8

#endregion